// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

//go:build !runOnTask

package jobqueue

import (
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
)

func TestWriteToDB1(t *testing.T) {

	gormDB, err := gorm.Open(sqlite.Open("file:unique_id?mode=memory&cache=shared"), &gorm.Config{})
	if err != nil {
		t.Fatalf("an error occurred while opening the database: %v", err)
	}
	t.Cleanup(func() {
		sqlDB, err := gormDB.DB()
		if err != nil {
			t.Fatalf("failed to get generic database object from GORM DB: %v", err)
		}
		_ = sqlDB.Close()
	})

	gormDB.Logger = gormDB.Logger.LogMode(4)

	manager := &Manager{database: gormDB}

	// Starte den DBSaver
	saver := NewJobSyncer(manager)

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job1", runner)

	scheduler := &TimeScheduler{
		Time: time.Now().Add(1 * time.Second),
		jobs: nil,
	}

	job.scheduler = scheduler
	saver.Sync(job)
	saver.Sync(job)

	manager.WaitSync()
	time.Sleep(1 * time.Second)
	var stats JobPersistence
	err = gormDB.First(&stats, "id = ?", job.GetID()).Error
	assert.Nil(t, err)

	assert.Equal(t, job.GetID(), stats.ID)
	assert.Equal(t, job.GetID(), stats.ID)
	manager.WaitSync()

}
