// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package jobqueue

import (
	"fmt"
)

var (
	ErrMissingDependency            = newUIError("missing dependency")
	ErrCycleDetected                = newUIError("cycle detected")
	ErrQueueEmpty                   = newUIError("queue is empty")
	ErrJobAlreadyExists             = newUIError("job already exists")
	ErrWorkerNotRunning             = newUIError("worker is not running")
	ErrMaxJobsReached               = newUIError("maximum number of jobs reached")
	ErrWorkerAlreadyRunning         = newUIError("worker is already running")
	ErrWorkerAlreadyAdded           = newUIError("worker is already added")
	ErrWorkerNotAdded               = newUIError("worker is not added")
	ErrNoWorkers                    = newUIError("no workers")
	ErrWorkerAlreadyStopped         = newUIError("worker is already stopped")
	ErrManagerAlreadyStopped        = newUIError("manager is already stopped")
	ErrManagerAlreadyRunning        = newUIError("manager is already running")
	ErrManagerNotRunning            = newUIError("manager is not running")
	ErrJobNotScheduled              = newUIError("job is not scheduled")
	ErrCronNotInitialized           = newUIError("cron is not initialized")
	ErrCPUPercentage                = newUIError("cpu percentage must be between 0 and 100")
	ErrIntervalIsZero               = newUIError("interval must be greater than 0")
	ErrUnknownRunnableType          = newUIError("unknown runnable type")
	ErrUnknownSchedulerType         = newUIError("unknown scheduler type")
	ErrUnsupportedDatabaseType      = newUIError("unsupported database type")
	ErrUnsupportedFileOption        = newUIError("unsupported file option")
	ErrUnsupportedCredentialType    = newUIError("unsupported credential type")
	ErrUnsupportedTransferDirection = newUIError("unsupported transfer direction")
	ErrInvalidData                  = newUIError("invalid data")
	ErrUnknownFormat                = newUIError("unknown format")
	ErrFailedToCreateTempFile       = newUIError("failed to create temp file")
	ErrFailedToWriteTempFile        = newUIError("failed to write temp file")
	ErrJobAlreadyScheduled          = newUIError("job already scheduled")
	ErrNoDatabaseConnection         = newUIError("no database connection")
	ErrSchedulerNotSet              = newUIError("scheduler is not set")
	ErrJobNotActive                 = newUIError("job is not active")
	ErrJobAlreadyActive             = newUIError("job is already active")
	ErrChannelAlreadyClosed         = newUIError("the channel is already closed")
	ErrUnknownScheduleType          = newUIError("unknown schedule type")
	ErrCannotLoadStatsFromDatabase  = newUIError("errors while loading stats from database")
	ErrInvalidTime                  = newUIError("invalid time")
	ErrSchedulerMisconfiguration    = newUIError("scheduler misconfiguration")
	ErrMaxRetriesReached            = newUIError("maximum number of retries reached")
	ErrTimeoutReached               = newUIError("timeout reached")
	ErrFailedToCreate               = newUIError("failed to create")
	ErrFailedToQueryExistingJob     = newUIError("failed to query an existing job")
	ErrFailedToSaveJob              = newUIError("failed to save a job")
	ErrParameterIsNil               = newUIError("parameter is nil")
	ErrJobIDEmpty                   = newUIError("job ID is empty")
	ErrManagerNotInitialized        = newUIError("manager is not initialized")
	ErrScheduleTimeIsInThePast      = newUIError("scheduled time is in the past")
)

// UIError is an error that can be displayed to the user
type UIError struct {
	msg string
}

func newUIError(msg string, args ...any) *UIError {
	return &UIError{msg: fmt.Sprintf(msg, args...)}
}

func (e *UIError) Error() string {
	return e.msg
}
